Imports System
Imports System.Text.RegularExpressions
Imports System.Web.UI.WebControls
Imports System.Web.UI
Imports System.Web
Imports System.Xml
Imports System.IO

Public Class CensorColumn
    Inherits DataGridColumn

    Public Overrides Sub InitializeCell(ByVal cell As TableCell, ByVal columnIndex As Integer, ByVal itemType As ListItemType)
        MyBase.InitializeCell(cell, columnIndex, itemType)

        Select Case itemType
            Case ListItemType.AlternatingItem, ListItemType.Item, ListItemType.SelectedItem
                AddHandler cell.DataBinding, AddressOf Me.PerformDataBinding
        End Select

    End Sub

    Private Sub PerformDataBinding(ByVal sender As Object, ByVal e As EventArgs)
        Dim cell As TableCell = sender
        Dim gridItem As DataGridItem = cell.NamingContainer
        Dim dataItem As Object = gridItem.DataItem

        If Me.DataField <> String.Empty Then
            cell.Text = PerformCensorship(DataBinder.Eval(dataItem, Me.DataField))
        End If
    End Sub


    Private Function PerformCensorship(ByVal censorMe As String) As String
        'If no CensorFile has been specified, return censorMe string
        If Me.CensorFile = String.Empty Then
            Return censorMe
        End If

        'Convert the CensorFile path into a physical path
        Dim filepath As String = HttpContext.Current.Server.MapPath(Me.CensorFile)

        'Make sure the file exists
        If Not File.Exists(filepath) Then
            Throw New Exception("File " & filepath & " does not exist!")
            Return censorMe
        End If

        'Read the XML file's contents
        Dim censorDoc As New XmlDocument()
        censorDoc.PreserveWhitespace = True
        censorDoc.Load(filepath)

        Dim root As XmlNode = censorDoc.FirstChild

        Dim findNodes As XmlNodeList = root.SelectNodes("/censors/censor/find")
        Dim replaceNodes As XmlNodeList = root.SelectNodes("/censors/censor/replace")

        Dim re As Regex
        Dim results As String = censorMe
        Dim i As Integer
        For i = 0 To findNodes.Count - 1
            re = New Regex("\b" & findNodes.Item(i).InnerText & "\b", RegexOptions.IgnoreCase)
            results = re.Replace(results, replaceNodes.Item(i).InnerText)
        Next

        Return results
    End Function



    Private m_dataField As String
    Private m_censorFile As String

    Public Property CensorFile() As String
        Get
            Return Me.m_censorFile
        End Get
        Set(ByVal Value As String)
            Me.m_censorFile = Value
        End Set
    End Property

    Public Property DataField() As String
        Get
            Return Me.m_dataField
        End Get
        Set(ByVal Value As String)
            Me.m_dataField = Value
        End Set
    End Property
End Class
